/************************************************************************/
/*  main.c                                                              */
/*  author: Marius Mnch <marius-muench@gmx.net>                        */
/*  This is the main file for the security module                       */
/*  The used basis is the nuts_ntnu/sat-sw repository as of 23/06/14    */
/*  Appended directories are:                                           */
/*  -security:	Contains files for the main functionality               */
/*  -debugio:	Contains files for debugging output (VirtualCom via USB)*/
/*				It provides stdlib like functions for io				*/
/************************************************************************/


#include <asf.h>

#include "FreeRTOS.h"
#include "task.h"
#include "security/security.h"
#include "simulation/link_simulation.h"

void blinkerTask(void);

void blinkerTask(void) {
	while(1) {
		gpio_toggle_pin(AVR32_PIN_PB03);
		vTaskDelay(1000);
	}
}


int main(void) {
	struct pll_config pllcfg;
	pm_switch_to_osc0(&AVR32_PM, 12000000, AVR32_PM_OSCCTRL0_STARTUP_2048_RCOSC);
	pll_config_init(&pllcfg, PLL_SRC_OSC0, 1, 96000000 / FOSC0);
	pll_enable(&pllcfg, 0);
	pll_wait_for_lock(0);
	sysclk_set_prescalers(1, 3, 1);	// CPU clock is 48MHz, PBA is 12MHz, PBB is 48MHz
	sysclk_set_source(SYSCLK_SRC_PLL0);
	
	flashc_set_wait_state(1); // 1WS on flash controller because SYSCLK > 30MHz
	pm_switch_to_clock(&AVR32_PM, AVR32_PM_MCSEL_PLL0);
	
	gpio_configure_pin(AVR32_PIN_PB03, GPIO_DIR_OUTPUT);

	
	stdio_usb_init();
	security_init(2);

	
	xTaskCreate(&simulation_link_task, (const signed portCHAR *)"UplinkSimulation Task", configMINIMAL_STACK_SIZE+MAX_PACKET_SIZE, NULL, tskIDLE_PRIORITY+1, NULL);
	xTaskCreate(&simulation_beacon_task, (const signed portCHAR *)"Simulation Timestamp Task", configMINIMAL_STACK_SIZE+512, NULL, tskIDLE_PRIORITY+1, NULL);

	
	xTaskCreate((TaskFunction_t)blinkerTask, "Blinker task", configMINIMAL_STACK_SIZE+512,NULL, 1, NULL);
	vTaskStartScheduler();


	return 42;// should not get here
}
