#include "rtc.h"
#include "i2c.h"

void rtc_init(void)
{
	rtc_set_time(0, 1, 1, 0, 0, 0);
}

int32_t rtc_get_timestamp(void)
{
	struct tm time = { 0 };
	time.tm_year = rtc_get_year() + 100;
	time.tm_mon = rtc_get_month() - 1;
	time.tm_mday = rtc_get_day();
	time.tm_hour = rtc_get_hour();
	time.tm_min = rtc_get_minute();
	time.tm_sec = rtc_get_second();
	return mktime(&time);
}

void rtc_set_timestamp(time_t timestamp)
{
	struct tm *time = localtime(&timestamp);
	rtc_set_time(
		time->tm_year - 100,
		time->tm_mon + 1,
		time->tm_mday,
		time->tm_hour,
		time->tm_min,
		time->tm_sec);
}

void rtc_set_time(uint8_t year, uint8_t month, uint8_t day, uint8_t hour, uint8_t minute, uint8_t second)
{
	rtc_set_second(second);
	rtc_set_minute(minute);
	rtc_set_hour(hour);
	rtc_set_day(day);
	rtc_set_month(month);
	rtc_set_year(year);
}

uint8_t rtc_get_second(void)
{
	uint32_t data = i2c_read(RTC_ADDRESS, 0x00, 1, 1);
	return ((data & 0x70) >> 4) * 10 + (data & 0xf);
}

uint8_t rtc_get_minute(void)
{
	uint32_t data = i2c_read(RTC_ADDRESS, 0x01, 1, 1);
	return ((data & 0x70) >> 4) * 10 + (data & 0xf);
}

uint8_t rtc_get_hour(void)
{
	uint32_t data = i2c_read(RTC_ADDRESS, 0x02, 1, 1);
	uint8_t hours = data & 0xf;
	if (data & 0x20) {
		return hours + 20;
	} else if (data & 0x10) {
		return hours + 10;
	} else {
		return hours;
	}
}

uint8_t rtc_get_day(void)
{
	uint32_t data = i2c_read(RTC_ADDRESS, 0x04, 1, 1);
	return ((data & 0x30) >> 4) * 10 + (data & 0xf);
}

uint8_t rtc_get_month(void)
{
	uint32_t data = i2c_read(RTC_ADDRESS, 0x05, 1, 1);
	return ((data & 0x10) >> 4) * 10 + (data & 0xf);
}

uint8_t rtc_get_year(void)
{
	uint32_t data = i2c_read(RTC_ADDRESS, 0x06, 1, 1);
	return ((data & 0xf0) >> 4) * 10 + (data & 0xf);
}	

void rtc_set_second(uint8_t second)
{
	i2c_write_byte(RTC_ADDRESS, 0x00, (second / 10) << 4 | (second % 10));
}

void rtc_set_minute(uint8_t minute)
{
	i2c_write_byte(RTC_ADDRESS, 0x01, (minute / 10) << 4 | (minute % 10));
}

void rtc_set_hour(uint8_t hour)
{
	uint8_t data = hour % 10;
	if (hour >= 20) {
		data |= 0x20;
	} else if (hour >= 10) {
		data |= 0x10;
	}
	i2c_write_byte(RTC_ADDRESS, 0x02, data);
}

void rtc_set_day(uint8_t day)
{
	i2c_write_byte(RTC_ADDRESS, 0x04, (day / 10) << 4 | (day % 10));
}

void rtc_set_month(uint8_t month)
{
	i2c_write_byte(RTC_ADDRESS, 0x05, (month / 10) << 4 | (month % 10));
}

void rtc_set_year(uint8_t year)
{
	i2c_write_byte(RTC_ADDRESS, 0x06, (year / 10) << 4 | (year % 10));
}