/*
* security.c
*
* Created: 28.11.2013 19:18:20
*  Author: nsr
*/

#include <stdio.h>
#include "FreeRTOS.h"
#include "task.h"
#include "i2c.h"
#include "rtc.h"
#include "polarssl/md5.h"
#include "security.h"
#include <string.h>


int mode;

void security_init(int init_mode)
{
	i2c_init();
	rtc_init();
	mode= init_mode;	
}

int security_set_mode(int new_mode){
	if (new_mode+1 == mode || new_mode-1 == mode || new_mode >= SECURITY_MODE_OFF || new_mode <= SECURITY_MODE_EXTENDED_AUTH){
		mode = new_mode;
		return 1;
	}
	return -1;
}

/* The core interface to the security module
 * Params:
 * inbuf: Pointer to the incoming buffer containing the packet being validated
 * outbuf: Pointer where the output shall be stored
 * *cb_succes: Callback function which shall be called to process the data - format: cb_success(char *data, char *outbuf, int datalen)
 * *cb_fail: Callback function which shall be called for error handling - format: cb_fail(int errorcode, char *outbuf)
*/
int security_validate_and_respond(char *inbuf, char **outbuf, int (*cb_success)(char *, char **, unsigned short), int (*cb_fail) (int, char **)){
	unsigned short datalen;
	char *data, response;
	int validate;
	unsigned short response_length;

	validate = security_validate(inbuf);

	if(validate<0){
		cb_fail(validate, outbuf);
		return validate;
	}
	
	datalen = security_extract_data_length(inbuf);
		

	response_length = cb_success(&inbuf[PROTO_HEADERLEN],outbuf,datalen);



//this function is still buggy FUUUUUUU
if (mode> SECURITY_MODE_BASIC_AUTH){
	char hmac[PROTO_HMACLEN];
	memcpy(&hmac,&inbuf[PROTO_POS_HMAC],PROTO_HMACLEN);
	security_response(outbuf,response_length,hmac);
}
	return 1;
}

static void security_response(char **outbuf, unsigned short outlen, char * hmac){	 
	*outbuf = realloc(*outbuf,outlen+PROTO_HEADERLEN);
	if (*outbuf){
		
		memmove(*outbuf+PROTO_HEADERLEN,*outbuf,outlen);
		memset(*outbuf+PROTO_POS_TIMESTAMP,'.',sizeof(int32_t));
		memcpy(*outbuf+PROTO_POS_HMAC,hmac,PROTO_HMACLEN);
		memcpy(*outbuf+PROTO_POS_DATALEN,outlen,sizeof(unsigned short));
	}
	//realloc failed, dont change anything
}


static unsigned short security_extract_data_length(char *inbuf){
	unsigned short datalen;
	memcpy(&datalen,&inbuf[PROTO_POS_DATALEN], sizeof(short));

	return datalen;
}

static int security_validate(char *inbuf){ 
	unsigned char hmac_recv[PROTO_HMACLEN];
	unsigned char hmac_calc[PROTO_HMACLEN];
	unsigned short datalen;
	int32_t timestamp_recv, time;
	if(mode > SECURITY_MODE_OFF){



		datalen = security_extract_data_length(inbuf);
		if(datalen < 1 || datalen > MAX_PACKET_SIZE-PROTO_HEADERLEN){
			return VALIDATE_INVALID_DATALEN;
		}

		if (mode>SECURITY_MODE_NO_TIME){
			memcpy(&timestamp_recv,&inbuf[PROTO_POS_TIMESTAMP], sizeof(int32_t));
			time = rtc_get_timestamp();
			if( time > timestamp_recv || time+TOLERANCE < timestamp_recv)
			return VALIDATE_INVALID_TIME;
		}
		
		memcpy(&hmac_recv,&inbuf[PROTO_POS_HMAC],PROTO_HMACLEN);
		memset(&inbuf[PROTO_POS_HMAC],0,PROTO_HMACLEN);
		md5_hmac((const unsigned char *) &HMAC_KEY, sizeof(HMAC_KEY)-1, inbuf, datalen+PROTO_HEADERLEN, hmac_calc);
		memcpy(&inbuf[PROTO_POS_HMAC],&hmac_recv,PROTO_HMACLEN);
		if (memcmp(hmac_calc,hmac_recv,PROTO_HMACLEN)){
			return VALIDATE_INVALID_HMAC;
		}
		
		
		return VALIDATE_SUCCESS;
	}
	return VALIDATE_SECURITY_DISABLED;
}