#!/usr/bin/perl
use warnings;
use strict;
use threads;
use threads::shared;
use Digest::HMAC_MD5 qw(hmac_md5 hmac_md5_hex);
use IO::All;
use IO::Socket::INET;
use Time::HiRes qw(usleep);

$|++;
my $ip = shift;
my $uplinkPort = shift;
my $downlinkPort = shift;
my $beaconPort = shift;
my $proto = 'tcp';
my $response = "";

my $dir = './out';
my $timestamp : shared = undef;
my $key = "ABCD";

my $sent_packages = 0;
my $fingerprint_counter = 0;


my $uplinkSocket = IO::Socket::INET->new(
	PeerHost => $ip,
	PeerPort => $uplinkPort,
	Proto => $proto,
	Timeout => 100,
) or die "Unable to connect to uplink at $ip:$uplinkPort - $!\n";

my $downlinkSocket = IO::Socket::INET->new(
	PeerHost => $ip,
	PeerPort => $downlinkPort,
	Proto => $proto,
	Timeout => 10,
) or die "Unable to connect to downlink at $ip:$downlinkPort - $!\n";

my $beaconSocket = IO::Socket::INET->new(
	PeerHost => $ip,
	PeerPort => $beaconPort,
	Proto => $proto,
	Timeout => 10,
) or die "Unable to connect to beacon link at $ip:$beaconPort - $!\n";

my $recv_t = threads->new(\&recvThread)->detach();
	while(!defined($timestamp)){}

my $incTimestampt_t = threads->new(\&incTimestampThread)->detach();

opendir (DIR, $dir) or die $!;
my @files = sort readdir(DIR); 
while (my $file = shift @files) {
	my ($data, $recv, $response);
	my $ts = $timestamp;
	my $replay_counter = -1;

	next unless (-f "$dir/$file");

	my $ofile = `./create_nap_header.pl $dir/$file $ts $key`;
	my $saved_hmac = `./get_nap_hmac.pl $ofile`;
	open FILE, "<", $ofile or die $!;
	{ 
		local $/ ; 
		$data = <FILE> ;  
	}
	close FILE;

	print "Sending: $ofile:  \n";
	$sent_packages++;

	print "Packet Length: ", length($data), "\n";

#Goto Label for replay testing
REPLAY:
	#Simulate uplink transmission delay
	my $transdelay = int(length($data)* 8 / 9600*1000*1000);
	usleep($transdelay);

	$uplinkSocket->send($data);
	$downlinkSocket->recv($recv,1024);
	
	if($recv =~ /\[RESPONSE\]\s(.*)/){
		$response = $1;
		my $respfile = "./in/tmp";
		open FILE, "+>", $respfile or die $!;
		print FILE $response;
		close FILE;

		print $recv;

###Only required for testing extended auth#################
#		my $recv_hmac = `./get_nap_hmac.pl $respfile`;         
#		if ($recv_hmac eq $saved_hmac){        
#			print "[ENAP] HMAC matches!\n";    
#			$fingerprint_counter++;    
#		}else{                                 
#			print unpack 'H*' ,$response;      
#			$recv =~ s/[^[:print:]\n]+/./g;    
#			print "$recv\n\n";                 
#		}                                      
#	}                                          
#}                                             
#print "[ENAP RESULT] $fingerprint_counter / $sent_packages"; 
############################################################

###Normal excution flow###
	}
#	Simulate donmlink transmission delay
#	$transdelay = int(length($response)* 8 / 9600*1000*1000);
#	usleep($transdelay);

###Only required for testing immediate replay attacks######
#	if($recv !~ /\[DEBUG\].*-/){
#			$replay_counter ++;
#			goto REPLAY;
#	}
#	if($replay_counter == -1 ){ print $recv};
#	print "Number of succesful replays: $replay_counter\n";
###########################################################
}

$uplinkSocket->close();
$downlinkSocket->close();
$beaconSocket->close();
closedir(DIR);


sub recvThread{
	my $response;
	while($beaconSocket->connected){
		$beaconSocket->recv($response,4096);
		$response =~ /.*?(\d{9,})/;
		$timestamp = $1+4;

	}
}

sub incTimestampThread{
	while(1){
		$timestamp += 1;
		sleep(1);
	}
}