#!/usr/bin/perl

use warnings;
use strict;
use Win32::SerialPort;
use IO::Socket::INET;
use threads;
use IO::Select;

$|++;

if($#ARGV < 3){
	die "Usage: $0 <serial port> <uplink port> <downlink port> <beacon link port>\n";
}
my $serialPort = shift;
my $uplinkPort = shift;
my $downlinkPort = shift;
my $beaconPort = shift;
my $PortObj = Win32::SerialPort->new($serialPort) or die "Failed to open $serialPort\n";
print "[*] Established serial connection\n";

my $uplinkServer = new IO::Socket::INET(
	LocalHost => '0.0.0.0',
	Proto => "tcp",
	LocalPort => $uplinkPort,
	Listen => 1,
	Timout => 10,
	reuse => 1,
) or die "Failed to open Socket on port $uplinkPort: $!\n";

my $downlinkServer = new IO::Socket::INET(
	LocalHost => '0.0.0.0',
	Proto => "tcp",
	LocalPort => $downlinkPort,
	Listen => 1,
	Timout => 10,
	reuse => 1,
) or die "Failed to open Socket on port $downlinkPort: $!\n";

my $beaconServer = new IO::Socket::INET(
	LocalHost => '0.0.0.0',
	Proto => "tcp",
	LocalPort => $beaconPort,
	Listen => 1,
	Timout => 10,
	reuse => 1,

) or die "Failed to open Socket on port $beaconPort: $!\n";

print "[*] Waiting for client to connect\n";
my $uplinkClient = $uplinkServer->accept;
my $downlinkClient = $downlinkServer->accept;
my $beaconClient = $beaconServer->accept;
my $peerAddr = $uplinkClient->peerhost();

print "[+] Client connected from $peerAddr\n";
my $thread_1 = threads->new(\&serialThread)->detach();

while (1){
	my ($in, $pre);
	$uplinkClient->recv($in,4096);
	last if (length($in)==0);
	$pre = addPseudoNetworkLayer($in);
	$PortObj->write($pre);
	$PortObj->write($in);
}


print "[-] Connection closed, Exiting...\n";

$uplinkClient->close();
$uplinkServer->close();
$downlinkClient->close();
$downlinkServer->close();
$beaconClient->close();
$beaconServer->close();


sub addPseudoNetworkLayer{
	my $in = shift;
	my $out = pack "n", length($in);
	return $out;
}

sub serialThread{
	$PortObj->user_msg(1);
	$PortObj->error_msg(1);
	$PortObj->databits(8);
	$PortObj->baudrate(115200);
	$PortObj->parity("none");
	$PortObj->stopbits(1);	
	$PortObj->buffers(4096, 4096);  
	$PortObj->read_interval(10);  
	$PortObj->read_char_time(5);     
	$PortObj->read_const_time(10); 
	$PortObj->write_settings;
	my $response = "";

	while(1){
		$response .= $PortObj->input;
		if ($response =~ /\n/){
			if ($response =~ /\[RESPONSE\](.*)/ ){
				$downlinkClient->send($response);
				$response = "";					
			}
			
			if ($response =~ /\[BEACON\](.*)/ ){
				$beaconClient->send($1);
				$response =~ s/\[BEACON\](.*)\n//;
			}
			
		}
	}

}